-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DPS")

L:RegisterTranslations("enUS", function() return {
	["Damage per Second"] = true,
	["DPS"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage per Second"] = "초당 피해량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil
local data = nil
local broadcastdirty = nil
local dirty = nil
local seconds = nil
local damageDone = nil
local moduleName = "DPS"
local isInCombat = {}
local combatStartTime = {}
local lastShotTime = {}

local COMBAT_TIMEOUT = 4

local roster = AceLibrary("Roster-2.1")
local parser = ParserLib:GetInstance("1.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 37217 $"):sub(12, -3)) or 1
mod.version = 4
mod.hidePercentage = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType(moduleName, "hit", self.OnParserEvent)
	self:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	self:ScheduleRepeatingEvent( self.CheckPartyCombat, 1, self )
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes(moduleName)
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	
	if type(seconds) == "table" then
		seconds = self:del(seconds)
	end
	seconds = self:new()
	
	if type(damageDone) == "table" then
		damageDone = self:del(damageDone)
	end
	damageDone = self:new()
	dirty = nil
	broadcastdirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Damage per Second"] end
function mod:GetShortDisplayName() return L["DPS"] end
function mod:GetBroadcastData()
	broadcastdirty = nil
	return data[playerName]
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:IsBroadcastDirty()
	return broadcastdirty
end

function mod:OnRemoteData(sender, input)
	data[sender] = input
	dirty = true
end

local function getDuration(name)
	local duration = seconds[name] or 0
	if combatStartTime[name] then
		duration = duration + math.max((lastShotTime[name] or GetTime()) - combatStartTime[name], 1.5)
	end
	return math.max(duration, 1.5)
end

function mod:ResetDPSTimer(name)
	seconds[name] =  getDuration(name)
	
	combatStartTime[name] = nil
	lastShotTime[name] = nil
	
	self:UpdateDPS(name)
end

function mod:PLAYER_REGEN_ENABLED()
	self:ResetDPSTimer( playerName )
end

function mod:UpdateDPS(name)
	local newDPS = math.floor((damageDone[name] or 0) / getDuration(name))

	if name == playerName then
		broadcastdirty = newDPS - (data[playerName] or 0) > 0
		data[name] = newDPS
		dirty = true
	elseif not Violation:UserViolation( name, moduleName ) then
		self:OnRemoteData( name, newDPS )
	end

end

function mod:OnParserEvent( info )
	local name, amount, victim = info.source, info.amount, info.victim

	if not name or name == ParserLib_NONE or not amount or amount == 0 then
		return
	end

	local unitid
	if name == ParserLib_SELF then
		unitid = "player"
	else
		unitid = roster:GetUnitIDFromName( name )
	end
	if not unitid then return end

	-- We ignore split damage from known units to known units.
	-- this includes soullink damage from you to your pet.
	-- thanks rophy for pointing this out.
	if info.isSplit and victim then
		local victimid
		if victim == ParserLib_SELF then
			victimid = "player"
		else
			victimid = roster:GetUnitIDFromName( victim )
		end
		if victimid then return end
	end
	
	if name == ParserLib_SELF then
		name = playerName
	end
 	local newTime = GetTime()
	if lastShotTime[name] and lastShotTime[name] + COMBAT_TIMEOUT < newTime then
		mod:ResetDPSTimer(name)
	end
	
	combatStartTime[name] = combatStartTime[name] or newTime
	lastShotTime[name] = newTime
	damageDone[name] = ( damageDone[name] or 0 ) + amount
	
	mod:UpdateDPS(name)
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] and not UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = nil
			self:ResetDPSTimer( name )
		elseif isInCombat[name] and lastShotTime[name] and lastShotTime[name] + COMBAT_TIMEOUT < GetTime() then
			self:ResetDPSTimer(name)
		elseif not isInCombat[name] and UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
