-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Damage")

L:RegisterTranslations("enUS", function() return {
	["Damage done"] = true,
	["D"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage done"] = "입힌 피해량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil
local data = nil
local broadcastdirty = 0
local dirty = nil
local moduleName = "Damage"

local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 37202 $"):sub(12, -3)) or 1
mod.version = 4
mod.canMergeData = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType(moduleName, "hit", self.OnParserEvent)
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes(moduleName)
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	dirty = nil
	broadcastdirty = 0
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Damage done"] end
function mod:GetShortDisplayName() return L["D"] end
function mod:GetBroadcastData()
	local x = broadcastdirty
	broadcastdirty = 0
	return x
end

function mod:IsBroadcastDirty()
	return broadcastdirty > 0 and true or false
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:OnRemoteData(sender, input)
	data[sender] = (data[sender] or 0) + input
	dirty = true
end

function mod:OnParserEvent( info )
	local source, amount, victim = info.source, info.amount, info.victim

	if not source or source == ParserLib_NONE or not amount or amount == 0 then
		return
	end

	-- We ignore split damage from known units to known units.
	-- this includes soullink damage from you to your pet.
	-- thanks rophy for pointing this out.
	if info.isSplit and victim then
		local victimid
		if victim == ParserLib_SELF then
			victimid = "player"
		else
			victimid = roster:GetUnitIDFromName( victim )
		end
		if victimid then return end
	end

	if source == ParserLib_SELF then
		data[playerName] = data[playerName] + amount
		broadcastdirty = broadcastdirty + amount
		dirty = true
	else
		local unitid = roster:GetUnitIDFromName( source )
		if not unitid then return end
		if not Violation:UserViolation( source, moduleName ) then
			mod:OnRemoteData( source, amount )
		end
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end


