-- Note that this file actually contains two modules; Healing and Overhealing.

-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local HL = AceLibrary("AceLocale-2.2"):new("Violation: Healing")

HL:RegisterTranslations("enUS", function() return {
	["Healing done"] = true,
	["H"] = true,
} end)

HL:RegisterTranslations("koKR", function() return {
	["Healing done"] = "치유량",
} end)

local OHL = AceLibrary("AceLocale-2.2"):new("Violation: Overhealing")

OHL:RegisterTranslations("enUS", function() return {
	["Overhealing done"] = true,
	["OH"] = true,
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil

local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Locals for healing module
-----------------------------------------------------------------------

local healingData = nil
local healingBroadcastdirty = 0
local healingDirty = nil
local healingModuleName = "Healing"

-----------------------------------------------------------------------
-- Locals for overhealing module
-----------------------------------------------------------------------

local overhealingData = nil
local overhealingBroadcastdirty = 0
local overhealingDirty = nil
local overhealingModuleName = "Overhealing"

local overhealMod

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local healingMod = Violation:NewModule(healingModuleName, "AceEvent-2.0")
healingMod.revision = tonumber(("$Revision: 37373 $"):sub(12, -3)) or 1
healingMod.version = 4
healingMod.canMergeData = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function healingMod:OnModuleRegister()
	playerName = UnitName("player")
	healingData = self:new()
end

function healingMod:OnModuleEnable()
	parser:RegisterInfoType(healingModuleName, "heal", self.OnParserEvent)
end

function healingMod:OnModuleDisable()
	parser:UnregisterAllInfoTypes(healingModuleName)
end

function healingMod:OnReset()
	if type(healingData) == "table" then
		healingData = self:del(healingData)
	end
	healingData = self:new()
	healingData[playerName] = 0
	healingBroadcastdirty = 0
	healingDirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function healingMod:GetRevision() return self.revision end
function healingMod:GetDisplayName() return HL["Healing done"] end
function healingMod:GetShortDisplayName() return HL["H"] end
function healingMod:GetBroadcastData()
	local x = healingBroadcastdirty
	healingBroadcastdirty = 0
	return x
end

function healingMod:IsBroadcastDirty()
	return healingBroadcastdirty > 0 and true or false
end

function healingMod:IsDirty()
	return healingDirty
end

function healingMod:ResetDirty()
	healingDirty = nil
end

function healingMod:OnRemoteData(sender, input)
	healingData[sender] = (healingData[sender] or 0) + input
	healingDirty = true
end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function healingMod:OnParserEvent(info)
	local source, amount, victim = info.source, info.amount, info.victim

	if not source or source == ParserLib_NONE or not amount or amount == 0 or not victim then
		return
	end

	local unitid
	if source == ParserLib_SELF then
		unitid = "player"
	else
		unitid = roster:GetUnitIDFromName( source )
	end
	if not unitid then -- we're only interested in friendlies
		return 
	end

	local victimid
	if victim == ParserLib_SELF then
		victimid = "player"
	else
		victimid = roster:GetUnitIDFromName( victim )
	end

	local overheal
	if source == ParserLib_SELF then
		overheal = getOverheal( victimid, amount )
		healingData[playerName] = healingData[playerName] + amount - overheal
		overhealingData[playerName] = overhealingData[playerName] + overheal
		healingBroadcastdirty = healingBroadcastdirty + amount - overheal
		overhealingBroadcastdirty = overhealingBroadcastdirty + overheal
		healingDirty = true
		overhealingDirty = true
	else
		if not Violation:UserViolation( source, moduleName ) then
			overheal = getOverheal( victimid, amount )
			healingMod:OnRemoteData( source, amount - overheal )
			overhealMod:OnRemoteData( source, overheal)
		end
	end
end

function healingMod:GetData(player)
	if player then
		return healingData[player]
	else
		return healingData
	end
end

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

overhealMod = Violation:NewModule(overhealingModuleName)
overhealMod.revision = tonumber(("$Revision: 37373 $"):sub(12, -3)) or 1
overhealMod.version = 4
overhealMod.canMergeData = true

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function overhealMod:OnModuleRegister()
	overhealingData = self:new()
end

function overhealMod:OnModuleEnable()
	self:RegisterEvent("Violation_ModuleDisabled")
	if not Violation:IsModuleActive(healingModuleName) then
		-- activate healing module since it is needed to get the overhealing values
		Violation:ToggleModule(healingModuleName, true)
	end
end

function overhealMod:OnModuleDisable()
end

function overhealMod:OnReset()
	if type(overhealingData) == "table" then
		overhealingData = self:del(overhealingData)
	end
	overhealingData = self:new()
	overhealingData[playerName] = 0
	overhealingBroadcastdirty = 0
	overhealingDirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function overhealMod:GetRevision() return self.revision end
function overhealMod:GetDisplayName() return OHL["Overhealing done"] end
function overhealMod:GetShortDisplayName() return OHL["OH"] end
function overhealMod:GetBroadcastData()
	local x = overhealingBroadcastdirty
	overhealingBroadcastdirty = 0
	return x
end

function overhealMod:IsBroadcastDirty()
	return overhealingBroadcastdirty > 0 and true or false
end

function overhealMod:IsDirty()
	return overhealingDirty
end

function overhealMod:ResetDirty()
	overhealingDirty = nil
end

function overhealMod:OnRemoteData(sender, input)
	overhealingData[sender] = (overhealingData[sender] or 0) + input
	overhealingDirty = true
end

function overhealMod:GetData(player)
	if player then
		return overhealingData[player]
	else
		return overhealingData
	end
end

function overhealMod:Violation_ModuleDisabled(name)
	if name == healingModuleName then
		Violation:ToggleModule(overhealingModuleName, false)
	end
end

